/* ----------------------------------------------------------------------
 * gc - program for determining the GC contents of a sequence.
 * Copyright (C) 2000 January Weiner III
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 ---------------------------------------------------------------------- */

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "genpak.h"
#include "gp_getopt.h"

#define PROGNAME "gp_gc"
#define VERSION "0.2"

/*
 *
 */


typedef struct {
	FILE* in ;
	FILE* out ;
	int onlymean ;
	int total ;
	int distro ; } opt_s ;

long ComputateGC(sekw *s) ;
int GCDistro(sekw* inseq, opt_s options);

int main(int argc, char *argv[])
{
	extern int optind ;
	sekw* inseq ;
	double squares=0.0, giece=0.0, mean=0.0 ;
	opt_s options ;

	long currgc=0, totalgc=0, currnt=0, totalnt=0;

	int c,nseq=0;
	char message[100] ;
	int errflg = 0 ;
	options.onlymean=FALSE;
	options.total=FALSE;
	options.distro=FALSE ;
	progname=argv[0] ;

	allwarnings=NULL ;

	while ((c = gp_getopt(argc, argv, "D:tHmvhqd")) != EOF)
		switch(c) {
		case 'D':
			if(sscanf(optarg,"%i",&options.distro) != 1)
				Error("Wrong argument for option -d") ;
			Warn("Making sequence GC distribution, window %i",options.distro) ;
			break ;
		case 't':
			Warn("Will calculate the total GC of all sequences") ;
			options.total=TRUE ;
			break ;
		case 'm':
			Warn("Will show only the mean gc of all sequences") ;
			options.onlymean=TRUE ;
			break ;
		case 'H':
			html=TRUE ;
			break ;
		case 'q':
			quiet=TRUE ;
			break ;
		case 'v':
			fprintf(stderr,"%s version %s\n",progname,VERSION) ;
			exit(0) ;
			break ;
		case 'd':
			debug=TRUE ;
			Warn("Running in debug mode") ;
			break ;
		case 'h':
			Help() ;
			break ;
		default:
			errflg++ ;
			break;
		}

	if(errflg) Error("Type '%s -h' for help",progname) ;

/* open the file pointer to read the sequences 
 * from: standard input or a file provided? */
	if(optind >= argc) options.in=stdin ;
	else options.in=TryOpen(argv[optind],"r") ;

/* opening the file pointer to write the output: 
 * standard output or file provided? */
	optind++ ;

	if(optind >= argc) options.out=stdout ;
	else options.out=TryOpen(argv[optind],"wb") ;

/* main loop */
	while((inseq=ReadSeqFrag(options.in,0,0,0)) != NULL) {

		
		currgc=ComputateGC(inseq) ;
		currnt=inseq->leng ;
		if(options.total) {
			totalgc += currgc ;
			totalnt += currnt ;
		} 

		giece=100 * ( (double) currgc) / ( (double) currnt)  ;
		nseq++ ;

		if(options.distro) GCDistro(inseq, options);

		if(options.onlymean) {
			mean += giece ;
			squares += giece*giece ;
		} else {
			if(!options.total && !options.distro) 
				fprintf(options.out,"%.2f %%  %s\n",giece,inseq->name) ;
		}

		free(inseq) ;
	}

	if(options.onlymean) {
		squares = pow(((squares - (mean*mean)/nseq)/(nseq-1)),0.5) ;
		mean= mean / (double) nseq ;
		fprintf(options.out,"%.2f %%=mean\n",mean) ;
		if(nseq > 1) 
			fprintf(options.out,", %.2f =SE\n",squares) ;
	}

	if(options.total) {
		giece=100 * ( (double) totalgc) / ( (double) totalnt)  ;
		fprintf(options.out,"%.2f\n",giece) ;
	}

	if(html) PrintAllWarnings(options.out) ;

	fclose(options.out) ;
	fclose(options.in) ;
	return(EXIT_SUCCESS);
}



/* Computing and printing GC distribution */

int GCDistro(sekw* s, opt_s o) {

	long i,w ;
	double gc ;
	char c ;
	long cnv[128] ;
	
	fprintf(o.out,"%s\t",s->name) ;

	for(i=0,gc=0;i<(s->leng-o.distro);i++) {

	cnv['A']=0 ; cnv['C']=0 ; cnv['G']=0 ; cnv['T']=0; cnv['U']=0;
		
		for(w=0;w<o.distro;w++) {
			c=toupper(s->sequ[i+w]) ;
			cnv[c]++ ;
		}

		gc=100.0*(cnv['G']+cnv['C'])/o.distro ;
		if(debug) Warn("position %i, [G][C]=%i,%i gc=%f window %i",
			i,cnv['G'],cnv['C'],gc,o.distro) ;
		fprintf(o.out,"%2.1f\t",gc) ;
	}

	fprintf(o.out,"\n") ;


	return EXIT_SUCCESS ;

}

/* Simple function returning the gc contents of a sequence */

long ComputateGC(sekw *s) {

	long i,dlugosc ;
	double cnv[128] ;
	long gc ;
	char c ;

	cnv['A']=0 ; cnv['C']=0 ; cnv['G']=0 ; cnv['T']=0; cnv['U']=0;

	dlugosc=strlen(s->sequ) ;
	for(i=0;i<dlugosc;i++) {
		c=toupper(s->sequ[i]) ;
		cnv[c]++ ;
	}

	gc=cnv['C'] + cnv['G'] ;

	if(debug) Warn("ComputateGC: GC=%i",gc) ;
	return(gc) ;

}


/* Standard mesage */

void Help()
{
printf("\n");
printf("%s version %s - determine the gc contents of sequence(s)",PROGNAME,VERSION);
printf("\n");
printf("  Usage:\n");
printf("     %s [options] [ input file ] [ output file ]\n",progname);
printf("\n");
printf("  Options:\n");
printf("     -t       : will calculate the total GC%% of all read sequences\n");
printf("     -m       : will show only mean GC%% of all read sequences\n");
printf("     -D value : for each sequence, a distribution of GC with window\n") ;
printf("                size equal to value is printed.\n") ;
printf("     -H       : run in HTML mode\n");
printf("     -q       : run in quiet mode\n");
printf("     -v       : print version information & exit\n");
printf("     -h       : print this help screen & exit\n\n");
exit(0);
}


			
