/* ----------------------------------------------------------------------
 * "shifting" gene positions
 * Copyright (C) 2000 January Weiner III
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 ---------------------------------------------------------------------- */

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>

#include "genpak.h"
#include "gp_getopt.h"

#define VERSION "0.1"
#define PROGNAME "gp_shift"

char *progname ;

typedef struct {
	FILE *in ;
	FILE *out ; 
	long start ;
	long end ;
	int relative_to_end ;} opt_s ;

/*
 *
 */

int main(int argc, char *argv[])
{
	extern int optind ;
	extern char *optarg ;
	int width=70 ; /* width with which the sequence gets formatted */
	opt_s options ;
	sekw *inseq, *outseq ;

	int c;
	char message[100] ;
	int errflg = 0 ;

	options.start=0 ;
	options.end=0 ;
	options.relative_to_end=FALSE ;

	progname=argv[0] ;

	while ((c = gp_getopt(argc, argv, "3s:e:Hqdvh")) != EOF)
		switch(c) {
		case 'H':
			html=TRUE ;
			if(debug) Warn("Assuming HTML type output") ;
			break ;
		case 'q':
			quiet=TRUE ;
			break ;
		case 'v':
			fprintf(stderr,"%s version %s\n",progname,VERSION) ;
			exit(0) ;
			break ;
		case 'd':
			debug=TRUE ;
			Warn("Running in debug mode") ;
			break ;
		case 'h':
			Help() ;
			break ;
		default:
			errflg++ ;
			break;
		}


	if(errflg) Error("Type '%s -h' for help",progname) ;

/* open the file pointer to read the sequences 
 * from: standard input or a file provided? */
	if(optind >= argc) options.in=stdin ;
	else options.in=TryOpen(argv[optind],"r") ;

/* opening the file pointer to write the output: 
 * standard output or file provided? */
	optind++ ;

	if(optind >= argc) options.out=stdout ;
	else options.out=TryOpen(argv[optind],"wb") ;
	
	if(html) PrintAllWarnings(options.out) ;
	fclose(options.out) ;
	fclose(options.in) ;
	return(0);
}

/* Standard mesage */

void Help()
{
printf("\n");
printf("%s, v. %s- no description yet",PROGNAME,VERSION);
printf("\n");
printf("  Usage:\n");
printf("     %s [options] [ input file ] [ output file ]\n",progname);
printf("\n");
printf("  Options:\n");
printf("     -v       : print version information & exit\n");
printf("     -h       : print this help screen & exit\n");
printf("     -q       : quiet, suppress error messages\n\n");
exit(0);
}


			
